Package: ChemmineR
Type: Package
Title: Cheminformatics Toolkit for R
Version: 3.52.0
Date: 2023-01-31
Author: Y. Eddie Cao, Kevin Horan, Tyler Backman, Thomas Girke
Maintainer: Thomas Girke <thomas.girke@ucr.edu>
Description: ChemmineR is a cheminformatics package for analyzing drug-like small molecule data in R. Its latest version contains functions for efficient processing of large numbers of molecules, physicochemical/structural property predictions, structural similarity searching, classification and clustering of compound libraries with a wide spectrum of algorithms. In addition, it offers visualization functions for compound clustering results and chemical structures.
License: Artistic-2.0
Depends: R (>= 2.10.0), methods
biocViews: Cheminformatics, BiomedicalInformatics, Pharmacogenetics,
        Pharmacogenomics, MicrotitrePlateAssay, CellBasedAssays,
        Visualization, Infrastructure, DataImport, Clustering,
        Proteomics,Metabolomics
Imports: rjson, graphics, stats, RCurl, DBI, digest, BiocGenerics, Rcpp
        (>= 0.11.0), ggplot2,grid,gridExtra,
        png,base64enc,DT,rsvg,jsonlite,stringi
Suggests: RSQLite, scatterplot3d, gplots, fmcsR, snow, RPostgreSQL,
        BiocStyle, knitr, knitcitations, knitrBootstrap, ChemmineDrugs,
        png,rmarkdown, BiocManager,bibtex
Enhances: ChemmineOB
URL: https://github.com/girke-lab/ChemmineR
VignetteBuilder: knitr
LinkingTo: Rcpp, BH
SystemRequirements: GNU make
git_url: https://git.bioconductor.org/packages/ChemmineR
git_branch: RELEASE_3_17
git_last_commit: 444df16
git_last_commit_date: 2023-04-25
Date/Publication: 2023-04-25
NeedsCompilation: yes
Packaged: 2023-04-25 20:32:43 UTC; biocbuild
